/**
 * @function doSignmod
 * @returns Promise
 */

"use strict";

var apemanpromise = require('apemanpromise'),
    extend = require('extend');

/** @lends doSignmod */
function doSignmod(User, Sign, data, digest, signed) {
    return User.findById(signed.user.id)
        .then(function (user) {
            if (!user) {
                return null;
            }
            return Sign.findAll({
                where: {user_id: user.get('id')}
            })
                .then(function (signs) {
                    return apemanpromise.all(
                        signs.map(function (sign) {
                            var password = data.password;
                            if (!password) {
                                return sign;
                            }
                            return digest(password, sign.salt).then(function (hash) {
                                return sign.update({
                                    hash: hash,
                                    signmod_at: new Date()
                                });
                            });
                        })
                    );
                })
                .then(function () {
                    delete data.password;
                    return user.update(data);
                })
                .then(function (user) {
                    return Sign.findById(signed.id).then(function (sign) {
                        var result = sign.toJSON();
                        result.user = user.toJSON();
                        return result;
                    })
                });
        });

}

module.exports = doSignmod;

