/**
 * @function doSignup
 * @returns Promise
 */
"use strict";

var extend = require('extend'),
    apemanpromise = require('apemanpromise');

/** @lends doSignup */
function doSignup(User, Sign, data, digest, signKeys, saltLength) {
    var salt = digest.newSalt(saltLength || 24);
    return digest(data.password, salt)
        .then(function (hash) {
            var user = extend(data);
            delete user.password;
            return User.create(user)
                .then(function (user) {
                    var userId = user.get('id');
                    var promises = signKeys.map(function (signKey) {
                        return Sign.create({
                            user_id: userId,
                            kind: signKey,
                            key: data[signKey],
                            salt: salt,
                            hash: hash,
                            signup_at: new Date()
                        });
                    });
                    return apemanpromise.all(promises).then(function (signs) {
                        return {
                            user: user,
                            signs: signs
                        };
                    });
                });
        });
}

module.exports = doSignup;
