/**
 * Test case for create.
 * Runs with nodeunit.
 */

var create = require('../lib/create.js'),
    freeport = require('freeport'),
    async = require('async'),
    fs = require('fs'),
    apemandb = require('apemandb'),
    apemanApp = require('apeman-app'),
    request = require('request');

var tmpDir = __dirname + '/../tmp',
    dbFile = tmpDir + '/testing-db01.db';

var db;
exports.setUp = function (done) {
    request = request.defaults({jar: true});
    db = apemandb({
        env: {
            DATABASE: 'testing-database02',
            USERNAME: 'FOO',
            PASSWORD: 'BAR',
            DIALECT: 'sqlite',
            STORAGE: dbFile
        },
        models: __dirname + '/../doc/mocks/models/*.json'
    });
    db.sync({force: true})
        .then(function (db) {
            done();
        });
};

exports.tearDown = function (done) {
    setTimeout(function () {
        fs.unlink(dbFile, function () {
            done();
        });
    }, 100); //Wait for flush.
};

exports['Do sign.'] = function (test) {
    var app = create(db, {
        passwordSpec: 'pdkdf2:120:184'
    });
    test.ok(app);
    freeport(function (err, port) {
        test.ifError(err);
        apemanApp('testing-app', {
            configuration: {
                $apps: {
                    'testing-app': {
                        '/': [
                            require('apeman-app-json')(),
                            require('apeman-app-format')(),
                            require('apeman-app-send')(),
                            require('apeman-app-header')(),
                            require('apeman-app-form')(),
                            require('apeman-app-session')(
                                db.models.Session, '1234qwer'
                            ),
                            require('apeman-app-signed')()
                        ],
                        '/api/sign': app
                    }
                }
            },
            port: port
        }, function (err, server) {
            test.ifError(err);
            var baseUrl = 'http://localhost:' + port + '/';
            async.series([
                function signgetFail(callback) {
                    request({
                        url: baseUrl + 'api/sign/signget',
                        method: 'GET'
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 404);
                        callback(err);
                    })
                },
                function signupFail(callback) {
                    request({
                        url: baseUrl + 'api/sign/signup',
                        method: 'POST',
                        form: {
                            account_name: 'foo',
                            password: 'foo_pswd'
                        }
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 400);
                        callback(err);
                    })
                },
                function signupSuccess(callback) {
                    request({
                        url: baseUrl + 'api/sign/signup',
                        method: 'POST',
                        form: {
                            account_name: 'foo',
                            email: 'foo@exampole.com',
                            password: 'foo_pswd'
                        }
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 200);
                        callback(err);
                    })
                },
                function signinSuccess(callback) {
                    request({
                        url: baseUrl + 'api/sign/signin',
                        method: 'POST',
                        form: {
                            key: 'foo',
                            password: 'foo_pswd'
                        }
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 200);
                        var data = JSON.parse(body);
                        test.equal(data.success, true);
                        callback(err);
                    })
                },
                function signgetSuccess(callback) {
                    request({
                        url: baseUrl + 'api/sign/signget',
                        method: 'GET'
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 200);
                        var signed = JSON.parse(body);
                        callback(err);
                    })
                },
                function signmodSucess(callback) {
                    request({
                        url: baseUrl + 'api/sign/signmod',
                        method: 'POST',
                        form: {
                            password: 'baz'
                        }
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 200);
                        callback(err);
                    })
                },
                function signoutSuccess(callback) {
                    request({
                        url: baseUrl + 'api/sign/signout',
                        method: 'POST'
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 200);
                        var data = JSON.parse(body);
                        test.ok(data.signed.signout_at);
                        callback(err);
                    })
                },
                function signinFail(callback) {
                    request({
                        url: baseUrl + 'api/sign/signin',
                        method: 'POST',
                        form: {}
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 400);
                        var data = JSON.parse(body);
                        var errors = data.errors;
                        test.equal(errors.key[0].reason, 'OBJECT_REQUIRED');
                        callback(err);
                    })
                },
                function signinSuccess(callback) {
                    request({
                        url: baseUrl + 'api/sign/signin',
                        method: 'POST',
                        form: {
                            key: 'foo',
                            password: 'baz'
                        }
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 200);
                        var data = JSON.parse(body);
                        test.equal(data.success, true);
                        callback(err);
                    })
                },
                function signdelSuccess(callback) {
                    request({
                        url: baseUrl + 'api/sign/signdel',
                        method: 'POST'
                    }, function (err, res, body) {
                        test.equal(res.statusCode, 200);
                        callback(err);
                    })
                }
            ], function (err) {
                test.ifError(err);
                server.close();
                test.done();
            });
        });
    });
};

